##
##  OSSP ase -- Affiliation Service Environment
##  Copyright (c) 2005 Ralf S. Engelschall <rse@engelschall.com>
##  Copyright (c) 2005 The OSSP Project <http://www.ossp.org/>
##
##  This file is part of OSSP ase, a service environment for managing
##  affiliations which can be found at http://www.ossp.org/pkg/tool/ase/.
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
##  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
##
##  ase.cgi.util.pl: utility functions
##

package ASE;
use base 'CGI::Application';
use strict;
use warnings;

#   helper function for determining self-referencing URL
sub url {
    my ($self, %attr) = @_;
    my $url = $self->param(-cgi)->url(-relative => 1);
    my $first = 1;
    foreach my $key (keys(%attr)) {
        my ($var, $val) = ($key, $attr{$key});
        $var =~ s/^-//s;
        $var =~ s/-/_/sg;
        $url .= ($first ? "?" : ";") . 
            URI::Escape::uri_escape($var) . "=" .
            URI::Escape::uri_escape($val);
        $first = 0;
    }
    return $url;
}

#   helper function for generating random string
sub makerandom {
    my ($self, $n, $set) = @_;
    my $rnd = '';
    $set =~ s/(.)-(.)/join('', $1..$2)/sge;
    for (my $i = 0; $i < $n; $i++) {
        $rnd .= substr($set, int(rand(length($set))), 1);
    }
    return $rnd;
}

#   helper function for determining remote hostname
sub remotehostname {
    my ($self) = @_;
    my $cgi = $self->param(-cgi);
    my $hostname = $cgi->remote_host();
    if ($hostname =~ m/^(\d+\.){3}\d+$/s) {
        my $iaddr = Socket::inet_aton($hostname);
        my $name = gethostbyaddr($iaddr, &Socket::AF_INET); 
        $hostname = $name if (defined($name) and $name ne '');
    }
    if ($hostname !~ m/^(?:[^.]+\.)+[^.]+$/s) {
        $hostname = "example.com";
    }
    return $hostname;
}

#   helper function for generating encrypted passwords
sub makepasswords {
    my ($self, $plaintext) = @_;
    my $pw = {};

    #   Plain Text
    $pw->{-plaintext} = $plaintext;

    #   generate random salt
    my $salt = $self->makerandom(8, "./0-9A-Za-z");

    #   Unix crypt(3) (both salted DES and MD5)
    $pw->{-cryptdes} = Crypt::UnixCrypt::crypt($pw->{-plaintext}, substr($salt, 0, 2));
    $pw->{-cryptmd5} = Crypt::PasswdMD5::unix_md5_crypt($pw->{-plaintext}, $salt);

    #   Plain MD5
    my $md5 = new Digest::MD5;
    $md5->add($pw->{-plaintext});
    $pw->{-md5} = $md5->hexdigest;
    undef $md5;

    #   Plain SHA-1
    my $sha1 = new Digest::SHA1;
    $sha1->add($pw->{-plaintext});
    $pw->{-sha1} = $sha1->hexdigest;
    undef $sha1;

    return $pw;
}

#   helper function for current time in YYYYMMDDHHMMSS format
sub datetime {
    my ($self, $t) = @_;
    $t = time() if (not defined($t));
    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday) = gmtime($t);
    return sprintf("%04d%02d%02d%02d%02d%02d", $year+1900, $mon+1, $mday, $hour, $min, $sec);
}

#   helper function for YYYYMMDDHHMMSS to more human format conversion
sub datetime2str {
    my ($self, $t) = @_;
    if ($t =~ m/^(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})$/) {
        $t = sprintf("%04d-%02d-%02d %02d:%02d:%02d", $1, $2, $3, $4, $5, $6); 
    }
    return $t;
}

#   helper function for writing a logbook entry
sub logbook ($$;@) {
    my ($self, $fmt, @args) = @_;
    my $dbi     = $self->param(-dbi);
    my $session = $self->param(-session);
    
    my $uuid = ($session->param("login") || undef);
    my $date = $self->datetime();
    my $text = (defined($args[0]) ? sprintf($fmt, @args) : $fmt);

    my $sth = $dbi->prepare(
        "INSERT INTO ase_logbook (lg_uuid, lg_date, lg_text) VALUES (?, ?, ?);"
    ) or die $dbi->errstr();
    $sth->execute($uuid, $date, $text) or die $dbi->errstr();
    return;
}

1;

