#!/usr/bin/env perl
##
##  OSSP ase -- Affiliation Service Environment
##  Copyright (c) 2005 Ralf S. Engelschall <rse@engelschall.com>
##  Copyright (c) 2005 The OSSP Project <http://www.ossp.org/>
##
##  This file is part of OSSP ase, a service environment for managing
##  affiliations which can be found at http://www.ossp.org/pkg/tool/ase/.
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
##  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
##
##  ase-auth.pl: authentication command line interface
##

#   language requirements
require 5.008;
use strict;
use warnings;

#   program configuration
our $cfg = {
    -prog_name   => 'OSSP ase',
    -prog_desc   => 'Affiliation Service Environment',
    -prog_vers   => '0.0.1',
    -prog_date   => '20-Feb-2005',
    -path_libdir => '.',
    -path_vardir => '.',
    -path_datdir => '.',
};

#   language extensions
use Getopt::Long;                       # from OpenPKG "perl"
use DBI;                                # from OpenPKG "perl-dbi"
use DBD::SQLite;                        # from OpenPKG "perl-dbi" (with_dbd_sqlite=yes)
use Crypt::UnixCrypt;                   # from OpenPKG "perl-crypto"
use Crypt::PasswdMD5;                   # from OpenPKG "perl-crypto"
use Digest::MD5;                        # from OpenPKG "perl-crypto"
use Digest::SHA1;                       # from OpenPKG "perl-crypto"

#   command line parsing
my $opt = {
    -help    => 0,
    -version => 0,
    -quiet   => 0,
    -type    => 'md5',
};
my $p = new Getopt::Long::Parser;
$p->configure("bundling");
$p->getoptions(
    'h|help'      => \$opt->{-help},
    'V|version'   => \$opt->{-version},
    'q|quiet'     => \$opt->{-quiet},
    't|type=s'    => \$opt->{-type},
) || die "option parsing failed";
undef $p;
if ($opt->{-help}) {
    print "Usage: ase-auth [options] <username> [<password>]\n" .
          "Available options:\n" .
          " -h,--help       print out this usage page\n" .
          " -V,--vesion     print version\n" .
          " -q,--quiet      quiet processing\n" .
          " -t,--type=TYPE  type of password\n";
    exit(0);
}
if ($opt->{-version}) {
    print "$cfg->{-prog_name} $cfg->{-prog_vers} ($cfg->{-prog_date})\n";
    exit(0);
}
if (@ARGV < 1 or @ARGV > 2) {
    print STDERR "ase-auth:ERROR: invalid number of arguments\n";
    exit(1);
}

#   determine parameters
my $username = $ARGV[0];
my $password = (defined($ARGV[1]) ? $ARGV[1] : $ENV{"PASSWORD"}) || "";
if ($password eq '') {
    print STDERR "ase-auth:ERROR: empty password\n";
    exit(1);
}

#   helper function for generating random string
sub makerandom {
    my ($n, $set) = @_;
    my $rnd = '';
    $set =~ s/(.)-(.)/join('', $1..$2)/sge;
    for (my $i = 0; $i < $n; $i++) {
        $rnd .= substr($set, int(rand(length($set))), 1);
    }
    return $rnd;
}

#   helper function for generating encrypted passwords
sub makepasswords {
    my ($plaintext) = @_;
    my $pw = {};

    #   Plain Text
    $pw->{-plaintext} = $plaintext;

    #   generate random salt
    my $salt = &makerandom(8, "./0-9A-Za-z");

    #   Unix crypt(3) (both salted DES and MD5)
    $pw->{-cryptdes} = Crypt::UnixCrypt::crypt($pw->{-plaintext}, substr($salt, 0, 2));
    $pw->{-cryptmd5} = Crypt::PasswdMD5::unix_md5_crypt($pw->{-plaintext}, $salt);

    #   Plain MD5
    my $md5 = new Digest::MD5;
    $md5->add($pw->{-plaintext});
    $pw->{-md5} = $md5->hexdigest;
    undef $md5;

    #   Plain SHA-1
    my $sha1 = new Digest::SHA1;
    $sha1->add($pw->{-plaintext});
    $pw->{-sha1} = $sha1->hexdigest;
    undef $sha1;

    return $pw;
}

#   establish database object
my $dbi = DBI->connect(
    "dbi:SQLite:dbname=ase.db", "", "",
    { RaiseError => 1, AutoCommit => 1 }
);
die "failed to connect to database \"ase.db\""
    if (not defined($dbi));

#   lookup member in database 
my $sth = $dbi->prepare(
    "SELECT * FROM ase_member" .
    " WHERE me_username = ? OR me_address = ?;")
    or die $dbi->errstr();
$sth->execute($username, $username)
    or die $dbi->errstr();
my $rec = $sth->fetchrow_hashref();
if (not defined($rec)) {
    print STDERR "Unknown login\n";
    exit(1);
}

#   disconnect from database
undef $sth;
$dbi->disconnect();

#   verify password
my $pw = &makepasswords($password);
my $type = $opt->{-type};
my $ok = 0;
eval "\$ok = \$pw->{-$type} eq \$rec->{'me_access_pw_$type'};";

#   give results
if (not $opt->{-quiet}) {
    print STDOUT ($ok ? "OK" : "BAD")."\n";
}
exit($ok ? 0 : 1);

